/*
**	Sambar Server JAVA Engine Implementation
**
**	Confidential Property of Tod Sambar
**	(c) Copyright Tod Sambar 2002
**	All rights reserved.
*/
package com.sambar.javaeng;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Map;
import javax.servlet.Filter;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;

public class SambarFilter
{
    protected Class 			fclass = null;
    protected Map 				initParams = null;
    protected Filter 			filter;
    protected Config 			config;
    protected HashMap 			pathSpecs;
	protected String			name;
	protected String			classname;
	protected SambarContext  	scontext;
    
    /* ---------------------------------------------------------------- */
    public SambarFilter(SambarContext scontext, String name, String className)
    {
        if ((name == null) || (name.length() == 0))
		{
            throw new IllegalArgumentException("SambarFilter: No name for " + 
				className);
		}

        if ((className == null) || (className.length() == 0))
		{
            throw new IllegalArgumentException(
				"SambarFilter: No classname for " + name);
		}

		this.name = name;
		this.classname = classname;
		this.scontext = scontext;
    }

    public void addPathSpec(String pathSpec)
    {
        if (pathSpecs == null)
            pathSpecs = new HashMap();
        pathSpecs.put(pathSpec, pathSpec);
    }
    
    public boolean isMappedToPath()
    {
        return (pathSpecs != null);
    }
    
    public boolean appliesTo(String path)
    {
		if (pathSpecs == null)
			return (false);

		if (path == null)
            return (false);

		ArrayList list = new ArrayList(pathSpecs.values());
		Iterator iter = list.iterator();
		while (iter.hasNext())
		{
			Map.Entry entry = (Map.Entry)iter.next();
			String pattern = (String)entry.getKey();
			
        	// Case 1 - Exact Match
        	if (pattern.equals(path))
            	return (true);

        	// Case 2 - Path Match ("/.../*")
        	if (pattern.equals("/*"))
            	return (true);      // Optimize a common case

	        if (pattern.endsWith("/*")) 
			{
           		String comparePath = path;
            	while (true) 
				{
               		if (pattern.equals(comparePath + "/*"))
                    	return (true);
                	int slash = comparePath.lastIndexOf('/');
                	if (slash < 0)
                   		break;
                	comparePath = comparePath.substring(0, slash);
            	}

            	continue;
        	}

        	// Case 3 - Extension Match
        	if (pattern.startsWith("*.")) 
			{
            	int slash = path.lastIndexOf('/');
            	int period = path.lastIndexOf('.');
            	if ((slash >= 0) && (period > slash))
				{
                	if (pattern.equals("*." + path.substring(period + 1)))
						return (true);
				}
        	}
        }

        return (false);
    }
    
    public void setInitParameter(String param,String value)
    {
        if (initParams == null)
            initParams = new HashMap(3);
        initParams.put(name, value);
    }

    public String getInitParameter(String param)
    {
        if (initParams == null)
            return null;
        return (String)initParams.get(param);
    }

    public Enumeration getInitParameterNames()
    {
        if (initParams == null)
            return Collections.enumeration(Collections.EMPTY_LIST);
        return Collections.enumeration(initParams.keySet());
    }

    public void start()
        throws Exception
    {
		ClassLoader loader = scontext.getClassLoader();

        fclass = (Class)loader.loadClass(classname);
        
        if (!javax.servlet.Filter.class
			.isAssignableFrom(fclass))
        {
            fclass = null;
            throw new IllegalStateException("Servlet class " + classname +
                                            " is not a javax.servlet.Filter");
        }

        filter = (Filter)fclass.newInstance();
        config = new Config();
        filter.init(config);
    }

    public void stop()
    {
        if (filter != null)
            filter.destroy();
        filter = null;
        config = null;
        fclass = null;
    }
    
    public Filter getFilter()
    {
        return this.filter;
    }
    
    class Config implements FilterConfig
    {
        public String getFilterName()
        {
            return SambarFilter.this.name;
        }

        public ServletContext getServletContext()
        {
            return SambarFilter.this.scontext;
        }
        
        public String getInitParameter(String param)
        {
            return SambarFilter.this.getInitParameter(param);
        }
    
        public Enumeration getInitParameterNames()
        {
            return SambarFilter.this.getInitParameterNames();
        }
    }
}
