/*
**		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 2002
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.IOException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.*;
import javax.servlet.http.*;

/* 
 * Filters
 *
 * Determine and call chains of Filters as configured in web.xml.
 */
public class SambarFilters
{
    private SambarContext	scontext = null;
    private Map 			filterMap = new HashMap();
    private List 			pathFilters = new ArrayList();

	public SambarFilters(SambarContext scontext)
	{
		this.scontext = scontext;
	}

    SambarFilter newFilter(String name, String className)
    {
        SambarFilter filter = new SambarFilter(scontext, name, className);
        filterMap.put(name, filter);
        return filter;
    }
    
    public SambarFilter getFilter(String name)
    {
        return (SambarFilter)filterMap.get(name);
    }

    public SambarFilter mapPathToFilter(String pathSpec,
                                        String filterName)
    {
        SambarFilter filter = (SambarFilter)filterMap.get(filterName);
        if (filter == null)
            throw new IllegalArgumentException("Unknown filter :" + filterName);
        
        if (!filter.isMappedToPath())
            pathFilters.add(filter);
        filter.addPathSpec(pathSpec);
        
        return filter;
    }    

    public synchronized void start()
        throws Exception
    {
        // Start filters
        Iterator iter = filterMap.values().iterator();
        while (iter.hasNext())
        {
            SambarFilter filter = (SambarFilter)iter.next();
            try
			{
				filter.start();
			}
            catch(Exception e)
			{
				throw e;
			}
        }
    }
    
    public synchronized void stop()
        throws InterruptedException
    {
        // Stop filters
        Iterator iter = filterMap.values().iterator();
        while (iter.hasNext())
        {
            SambarFilter filter = (SambarFilter)iter.next();
            filter.stop();
        }
    }
    
    public void handle(String pathInContext,
                       String pathParams,
                       ServletRequest req,
                       ServletResponse res)
         throws IOException
    {
        try 
		{
			FilterChain chain = new Chain(pathInContext);
			chain.doFilter(req, res);
		}
        catch(ServletException e)
        {
            Throwable th = e.getRootCause();
            if (th instanceof IOException)
                throw (IOException)th;
        }
    }

    private class Chain implements FilterChain
    {
        int nextPathFilter;
        int nextServletFilter;
        String pathInContext;
        List servletFilters;
        
        Chain(String pathInContext)
        {
            nextPathFilter=0;
            nextServletFilter=0;
            pathInContext=pathInContext;
        }
        
        public void doFilter(ServletRequest request, ServletResponse response)
            throws IOException, ServletException
        {
            // Find the next applicable path filter
            while (nextPathFilter < pathFilters.size())
            {
                SambarFilter sf=(SambarFilter)pathFilters.get(nextPathFilter++);
                if (sf.appliesTo(pathInContext))
                {
                    Filter filter = sf.getFilter();
                    filter.doFilter(request,response,this);
                    return;
                }
            }
        }   
    }
}
