/*
** 		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 1999
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.sambar.javaeng.SambarAPI;
import com.sambar.javaeng.SambarUtils;

public class SambarHandler 
{
	public static boolean 		traceServlets = false;
	private static Hashtable	scontextHash = null;
	private static Hashtable	genericServletHash = null;

	private static synchronized void setContext(long jeng, 
		SambarContext scontext)
	{
		if (scontextHash == null)
			scontextHash = new Hashtable();

		scontextHash.put("J" + jeng, scontext);
	}

	private static synchronized SambarContext getContext(long jeng)
	{
		if (scontextHash == null)
			return null;

		return (SambarContext)scontextHash.get("J" + jeng);
	}

	/* 
	** Called from the Sambar Server JavaEngine
	*/
	public static int init(long jeng)
	{
		SambarContext scontext = null;

		try 
		{
			scontext = new SambarContext(jeng);
			setContext(jeng, scontext);
		}
		catch(RuntimeException re) 
		{
			SambarAPI.logctx(jeng, SambarAPI.LOG_WARN,
				"SambarContext initialization failure: " + re.toString());
			return SambarAPI.FAIL;
		}

		String val = System.getProperty("traceServlets");
		if ((val != null) && (val.equalsIgnoreCase("true")))
			traceServlets = true;

		// Generic startup servlet specified?
		String name = SambarAPI.getGenericServlet(jeng);
		if (name != null)
		{
			try 
			{
				Servlet genericServlet = scontext.getServlet(name, false);
				if (genericServlet == null)
				{
					SambarAPI.logctx(jeng, SambarAPI.LOG_WARN,
						"GenericServlet '" + name + "' not found...");
					return SambarAPI.FAIL;
				}

				scontext.setGenericServlet(genericServlet);
			}
			catch(Exception ex) 
			{
				SambarAPI.logctx(jeng, SambarAPI.LOG_WARN,
					"GenericServlet '" + name + "' not found - " +
					ex.toString());
				return SambarAPI.FAIL;
			}

			// service() not called on this GenericServlet
		}

		try
		{
			// Load the Jasper JSP engine at startup.
			Servlet servlet = scontext.getServlet("org.apache.jasper.servlet.JspServlet", true);
		}
		catch (Exception e)
		{
			SambarAPI.logctx(jeng, SambarAPI.LOG_WARN,
				"Jasper JspServlet not found - " + e.toString());
		}

		return SambarAPI.SUCCEED;
	}

	public static void exit(long jeng)
	{
		SambarContext scontext = getContext(jeng);
		if (scontext == null)
		{
			SambarAPI.logctx(jeng, SambarAPI.LOG_ERROR,
				"SambarHandler exit failure; no context for engine");
			return;
		}

		try 
		{
			Servlet genericServlet = scontext.getGenericServlet();
			if (genericServlet != null)
				genericServlet.destroy();

			scontext.setGenericServlet(null);
		}
		catch(Exception ex) 
		{
			SambarAPI.logctx(jeng, SambarAPI.LOG_ERROR,
				"GenericServlet destroy failure - " +
				ex.toString());
		}

		scontext.destroyServlets();
		SambarAPI.logctx(jeng, SambarAPI.LOG_TRACE,
			"JavaEngine SambarHandler 'exit' method called...");
	}

	public static int handler(long req) 
	{
		String name = SambarAPI.getServletName(req);
		long jeng = SambarAPI.getJavaEngine(req);
		SambarContext scontext = getContext(jeng);
		if (scontext == null)
		{
			SambarAPI.logctx(jeng, SambarAPI.LOG_ERROR,
				"SambarHandler failure; no context for VHOST engine");
			return SambarAPI.FAIL;
		}

		if (traceServlets)
		{
			SambarAPI.log(req, SambarAPI.LOG_TRACE, "Servlet " + name + 
				" being called (J" + jeng + ")...");
		}

		Servlet servlet;
		SambarConnection sconn = new SambarConnection(req, scontext);

		try 
		{
			servlet = scontext.getServlet(name, true);
			if (servlet == null)
			{
				SambarAPI.log(req, SambarAPI.LOG_WARN, "Servlet " + name + 
					" not found...");
				sconn.sendError(HttpServletResponse.SC_NOT_FOUND, 
					"Servlet " + name + " not found...");
				sconn.close();
				return SambarAPI.FAIL;
			}
		}
		catch(Exception ex) 
		{
			SambarAPI.log(req, SambarAPI.LOG_WARN, "Servlet " + name + 
				" not found - " + ex.toString());
			sconn.sendError(HttpServletResponse.SC_NOT_FOUND, 
				"Servlet " + name + " not found...");
			sconn.close();
			return SambarAPI.FAIL;
		}

		// Save the class loader for use by the JSP engine
		Thread.currentThread().setContextClassLoader(scontext.getClassLoader());

		try 
		{
			servlet.service((ServletRequest)sconn, (ServletResponse)sconn);
		}
		catch(Exception ex) 
		{
			SambarAPI.log(req, SambarAPI.LOG_ERROR, "Servlet " + name + 
				" failed with exception - " + ex.toString());
			sconn.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, 
				"Servlet " + name + " raised exception...\n" +
				stack2string(ex));
			sconn.close();
			scontext.destroyServlet(name);
			return SambarAPI.FAIL;
		}

		sconn.close();
		return SambarAPI.SUCCEED;
  	}

	static public String stack2string(Exception e) 
	{
		try 
		{
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw);
			e.printStackTrace(pw);
			return "------\r\n" + SambarUtils.escapeHTML(sw.toString()) + 
				"------\r\n";
		}
		catch(Exception e2) 
		{
			return "no stack available.";
		}
	} 
}
